import socket
import tempfile
from typing import Optional, List

from divinegift import logger

try:
    from smb.SMBConnection import SMBConnection
    from smb.base import SharedFile
except ImportError:
    raise ImportError("pysmb isn't installed. Run: pip install -U pysmb")


class SmbClient:
    def __init__(self,
                 path: Optional[str] = None,
                 username: Optional[str] = None,
                 password: Optional[str] = None,
                 domain: Optional[str] = 'group.s7'):
        self.path: Optional[str] = path
        self.username: Optional[str] = username
        self.password: Optional[str] = password
        self.domain: Optional[str] = domain

        self.remote_name: Optional[str] = None
        self.service_name: Optional[str] = None
        self.share_path: Optional[str] = None

        self.conn: Optional[SMBConnection] = None

    def prepare_connection(self):
        '''
        Fill internal variables
        :return: None
        '''
        if self.path:
            path_arr = self.path.replace('\\', '/').split('/')[2:]
            self.remote_name = path_arr[0]
            self.service_name = path_arr[1]
            self.share_path = '/'.join(path_arr[2:])

    def set_path(self, path: str):
        self.path = path

    def set_credentials(self, username: str, password: str, domain='group.s7'):
        assert username, "Empty username isn't allowed"
        assert password, "Empty password isn't allowed"
        self.username = username
        self.password = password
        self.domain = domain

    def connect(self) -> SMBConnection:
        '''
        Connect to SMB
        :return: SMBConnection instance
        '''
        assert self.username, 'No username provided'
        assert self.password, 'No password provided'

        conn = SMBConnection(username=self.username,
                             password=self.password,
                             my_name=socket.gethostname(),
                             remote_name=self.remote_name,
                             domain=self.domain,
                             use_ntlm_v2=True,
                             is_direct_tcp=True)

        assert conn.connect(socket.gethostbyname(self.remote_name), 445)

        self.conn = conn
        return self.conn

    def download(self, filename, path_to='.') -> bool:
        '''
        Download file from SMB
        :param filename: filename
        :param path_to: path to store file in local system
        :return: True if file downloaded else False
        '''
        tmp_file, got_file = self.download_as_tmp(filename)
        if got_file:
            try:
                name = f'{path_to}/{filename}'
                with open(name, 'wb') as f:
                    f.write(tmp_file.read())
            except Exception as ex:
                got_file = False
            finally:
                if tmp_file is not None:
                    tmp_file.close()
        return got_file
    
    def download_as_tmp(self, filename) -> tuple[tempfile.TemporaryFile, bool]:
        '''
        Download file from SMB to temporary file
        :param filename: filename
        :return: tupe[temporary file from SMB, download success status]
        '''
        try:
            tmp_file = tempfile.TemporaryFile()
            self.conn.retrieveFile(self.service_name, f'{self.share_path}/{filename}', tmp_file)
            tmp_file.seek(0)
            return tmp_file, True
        except Exception as ex: 
            logger.log_err(f"Couldn't download file from {self.share_path}/{filename}")
            return None, False
    
    def upload(self, filename, path_from):
        '''
        Upload file to SMB
        :param filename: File name
        :param path_from: where file stored in local storage
        :return:
        '''
        try:
            name = f'{path_from}/{filename}'
            with open(name, 'rb') as f:
                b = self.conn.storeFile(self.service_name, f'{self.share_path}/{filename}', f)
        except Exception as ex:
            logger.log_err(f"Couldn't store file {self.share_path}/{filename}")
    
    def upload_tmp_file(self, filename, tmp_file):
        '''
        Upload tmp file to SMB
        :param filename: File name to save in remote storage
        :param tmp_file: temporary file to upload to SMB
        :return:
        '''
        try:
            _ = self.conn.storeFile(self.service_name, f'{self.share_path}/{filename}', tmp_file)
        except Exception as ex:
            logger.log_err(f"Couldn't store tmp file {self.share_path}/{filename}")
        
    def delete(self, filename):
        '''
        Delete file from SMB
        :param filename: File name
        :return:
        '''
        try:
            self.conn.deleteFiles(self.service_name, f'{self.share_path}/{filename}')
        except Exception as ex:
            logger.log_err(f"Couldn't delete file {self.share_path}/{filename}")

    def ls(self, subfolder=''):
        '''
        Print list of files to STDOUT
        :param subfolder: Subfolder where to look for files
        :return:
        '''
        file_list = self.get_list_files(subfolder)
        for file in file_list:
            print(file.filename)

    def get_list_files(self, additional_directory='') -> List[SharedFile]:
        '''
        Get list of files from SMB
        :param additional_directory: Subfolder where to look for files
        :return:
        '''
        try:
            file_list = self.conn.listPath(self.service_name, f'{self.share_path}/{additional_directory}')
            return [x for x in file_list if x.filename not in ('.', '..')]
        except Exception as ex:
            logger.log_err(f"Couldn't get list of files {self.share_path}/{additional_directory}")



from deprecation import deprecated

from divinegift import main, version
from divinegift import sender

import os
import logging
from logging.handlers import TimedRotatingFileHandler
import traceback

from typing import List, Dict, Optional


class Logger:
    def __init__(self):
        self.logger: Optional[logging.Logger] = None
        self.log_dir = None
        self.log_name = None
        self.formatter = logging.Formatter('%(levelname)-8s [%(asctime)s] %(message)s')
        self.sender = sender.Sender()

    def set_log_level_(self):
        self.set_log_level(**main.get_log_param(main.get_args()))

    def set_log_level(self, log_level: str, log_name: str = None, log_dir: str = './logs/', when: str = 'midnight',
                      interval: int = 1, backup_count: int = 7):
        """
            This set up log_level and name of logfile
            :param log_level: String with log_level (e.g. 'INFO')
            :param log_name: Name of file with logs
            :param log_dir: Directory which should keep logs
            :param when: When rotate log
            :param interval: How often
            :param backupCount: How many version need to keep
            :return: None
        """

        self.logger = logging.getLogger('Rotating')
        # Remove all existing handlers
        for handler in self.logger.handlers[:]:
            self.logger.removeHandler(handler)
        numeric_level = getattr(logging, log_level.upper(), None)
        if not isinstance(numeric_level, int):
            raise ValueError('Invalid log level: %s' % log_level)

        self.logger.setLevel(numeric_level)

        if log_name:
            if not os.path.exists(log_dir):
                os.makedirs(log_dir)
            handler = TimedRotatingFileHandler(os.path.join(log_dir, log_name), when=when, interval=interval,
                                               backupCount=backup_count, encoding='utf-8')
            self.log_name = log_name
            self.log_dir = log_dir
        else:
            handler = logging.StreamHandler()

        handler.setFormatter(self.formatter)
        self.logger.addHandler(handler)

    def log_debug(self, *args, separator: str = ' '):
        """
        Logging a debug message
        :param msg: Message which would be logged
        :return: None
        """
        if not self.logger:
            self.set_log_level_()
        self.logger.debug(separator.join([str(x) for x in args]))

    def log_info(self, *args, separator: str = ' '):
        """
        Logging a info
        :param separator: Separator
        :param msg: Message which would be logged
        :return: None
        """
        if not self.logger:
            self.set_log_level_()
        self.logger.info(separator.join([str(x) for x in args]))

    def log_warn(self, *args, separator: str = ' '):
        """
        Logging a warning
        :param separator: Separator
        :param msg: Message which would be logged
        :type msg: string
        :return: None
        """
        if not self.logger:
            self.set_log_level_()
        self.logger.warning(separator.join([str(x) for x in args]))

    def log_err(self, *args, separator: str = ' ', src: str = None, mode: List = None, channel: Dict = None):
        """
        Logging an error with monitoring if parameters were filled
        :param separator: Separator
        :param msg: Error message which will logged
        :param src: Source which raised error
        :type src: string
        :param mode: List of monitoring's mode (telegram, email, email_attach, slack)
        :type mode: list
        :param channel: Dict with parameters of monitoring (e.g. {'telegram': -1001343660695}
        :type channel: dict
        :return: None
        """
        if not self.logger:
            self.set_log_level_()
        self.logger.exception(separator.join([str(x) for x in args]))

        error_txt = f'''An error has occurred in {src}
Error text: {separator.join([str(x) for x in args])}\n{traceback.format_exc()}'''

        if mode:
            if 'telegram' in mode:
                self.sender.send_telegram(error_txt, chat_id=channel.get('telegram', -1001343660695))
            if 'slack' in mode:
                self.sender.send_slack(error_txt)
            if 'email' in mode:
                self.sender.send_mail(error_txt, f'{src} ERROR', **channel.get('email'))
            if 'email_attach' in mode:
                if self.log_dir and self.log_name:
                    log = main.get_list_files(self.log_dir, self.log_name, add_path=True)
                else:
                    log = []
                self.sender.send_mail(error_txt, f'{src} ERROR', attachments=log, **channel.get('email_attach'))

    def log_crit(self, *args, separator: str = ' '):
        if not self.logger:
            self.set_log_level_()
        self.logger.critical(separator.join([str(x) for x in args]))


@deprecated(deprecated_in='2.8.0', current_version=version, details='Use class Logger instead')
def log_debug(*args, separator: str = ' '):
    get_logger().log_debug(*args, separator=separator)


@deprecated(deprecated_in='2.8.0', current_version=version, details='Use class Logger instead')
def log_info(*args, separator: str = ' '):
    get_logger().log_info(*args, separator=separator)


@deprecated(deprecated_in='2.8.0', current_version=version, details='Use class Logger instead')
def log_warning(*args, separator: str = ' '):
    get_logger().log_warn(*args, separator=separator)


@deprecated(deprecated_in='2.8.0', current_version=version, details='Use class Logger instead')
def log_err(*args, separator: str = ' ', src: str = None, mode: List = None, channel: Dict = None):
    get_logger().log_err(*args, separator=separator, src=src, mode=mode, channel=channel)


@deprecated(deprecated_in='2.8.0', current_version=version, details='Use class Logger instead')
def log_crit(*args, separator: str = ' '):
    get_logger().log_crit(*args, separator=separator)


def set_loglevel(log_level: str, log_name: str = None, log_dir: str = './logs/',
                 when: str = 'midnight', interval: int = 1, backupCount: int = 7):
    get_logger().set_log_level(log_level, log_name, log_dir, when, interval, backupCount)


logger_ = Logger()


def get_logger():
    return logger_


if __name__ == '__main__':
    pass



from datetime import datetime
from typing import Optional
from divinegift import logger

from functools import wraps


class Timer:
    def __init__(self, logger_: Optional[logger.Logger] = None):
        self._st = datetime.now()
        self._sp: Optional[datetime] = None
        self.desc: Optional[str] = None
        self.logger = logger_ if logger_ else logger.Logger()

    def start(self, desc: Optional[str] = None):
        """
        :param desc: description which will be printed before time
        :return:
        """
        self._st = datetime.now()
        self.desc = desc

    def stop(self, format_: Optional[str] = None, print_: bool = True):
        """
        :param format_: format, maybe None, 's' - sec, 'm' - min, 'h' - hours
        :param print_: log it by logger or just print
        :return:
        """
        def pr(diff_, format_):
            if format_:
                diff__ = '{:0.5f}'.format(diff_)
            else:
                diff__ = diff_
            str_ = f'{f"[{self.desc}] " if self.desc else ""}{diff__} {format_}'
            if print_:
                print(str_)
            else:
                self.logger.log_info(str_)

        self._sp = datetime.now()
        diff = self._sp - self._st
        if not format_:
            pr(diff, '')
        elif format_ == 's':
            pr(diff.total_seconds(), 'sec')
        elif format_ == 'm':
            s = diff.total_seconds()
            m = s / 60
            if m < 0.00001:
                pr(s, 'sec')
            else:
                pr(m, 'min')
        elif format_ == 'h':
            s = diff.total_seconds()
            m = s / 60
            h = m / 60
            if h < 0.00001:
                if m < 0.00001:
                    pr(s, 'sec')
                else:
                    pr(m, 'min')
            else:
                pr(h, 'hr')


def exec_time(_func=None, *, format_: Optional[str] = None, print_: bool = True, desc: Optional[str] = None):
    """
    Decorator which shows execution time
    :param _func: Decorated function
    :param format_: format, maybe None, 's' - sec, 'm' - min, 'h' - hours
    :param print_: log it by logger or just print
    :param desc: description which will be printed before time
    """

    def _exec_time(func_):
        t = Timer()

        @wraps(func_)
        def wrapper(*args, **kwargs):
            t.start(desc)
            res = func_(*args, **kwargs)
            t.stop(format_, print_)
            return res
        return wrapper

    if _func:
        return _exec_time(_func)

    return _exec_time


if __name__ == '__main__':
    t = Timer()
    t.stop(format_='s', print_=True)

    @exec_time
    def foo():
        print('foo')

    @exec_time(format_='h', print_=True, desc='Test bar')
    def bar(str_):
        from time import sleep
        sleep(0.001)
        print(str_)

    foo()
    bar('var')
